/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ConfiguratorFrameDescriptor.java 20821 2011-02-02 16:59:06Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.configurationframe;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Collections;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JFileChooser;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Get;
import org.ow2.jonas.autostart.builder.Builder;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.fullwizard.FullWizardPanelDescriptor;
import org.ow2.jonas.autostart.parser.Parser;
import org.ow2.jonas.autostart.parseurl.ParseUrl;
import org.ow2.jonas.autostart.utility.Utility;

/**
 * This class instanciates ConfiguratorFrame.
 * @author skabore
 */
public class ConfiguratorFrameDescriptor extends FullWizardPanelDescriptor implements ActionListener, ListSelectionListener {

    /**
     * The identifier.
     */
    public static final String IDENTIFIER = "CONFIGURATION_PANEL";

    /**
     * The BuildSummaryFrame identifier.
     */
    private String buildSummaryFrameId;

    /**
     * The GettingJonasFieldFromWebsiteFrame identifier.
     */
    private String gettingJonasFieldFromWebsiteFrameId;

    /**
     * The GettingJonasFieldFromDiskFrame identifier.
     */
    private String gettingJonasFieldFromDiskFrameId;

    /**
     * The GettingJonasFieldFromZipFileFrame identifier.
     */
    private String gettingJonasFieldFromZipFileFrameId;

    /**
     * The RunningJonasOption identifier.
     */
    private String runningJonasOptionFrameId;

    /**
     * The LicenseFrame identifier.
     */
    private String licenseFrameId;

    /**
     * The builder librairy.
     */
    private Builder builder;

    /**
     * The ConfiguratorFrame.
     */
    private ConfiguratorFrame configuratorFrame;

    /**
     * The file separator.
     */
    private static final String FILE_SEPARATOR = System.getProperty("file.separator");

    /**
     * The callFromStraterFrame.
     */
    private boolean callFromStraterFrame;

    /**
     * The constructeur.
     * @param configuration the configuration librairy
     * @param stepId, the step identifier
     * @param steps, the required steps to create a starter
     */
    public ConfiguratorFrameDescriptor(final Configuration configuration, final int stepId, final int steps) {
        this.configuratorFrame = new ConfiguratorFrame(configuration, stepId, steps);
        this.configuratorFrame.addActionListener(this);
        this.configuratorFrame.addListSelectionListener(this);
        setPanelDescriptorIdentifier(IDENTIFIER);
        setPanelComponent(this.configuratorFrame);
        this.callFromStraterFrame = false;
    }

    /**
     * @return builder.
     */
    public Builder getBuilder() {
        return this.builder;
    }

    /**
     * @return buildSummaryFrameId.
     */
    public String getBuildSummaryFrameId() {
        return this.buildSummaryFrameId;
    }

    /**
     * Set the buildSummaryFrameId.
     * @param buildSummaryFrameId the new
     *        buildSummaryFrameIConfiguratorFrameDescriptord value.
     */
    public void setBuildSummaryFrameId(final String buildSummaryFrameId) {
        this.buildSummaryFrameId = buildSummaryFrameId;
    }

    /**
     * @return callFromStraterFrame.
     */
    public boolean isCallFromStraterFrame() {
        return this.callFromStraterFrame;
    }

    /**
     * Set the callFromStraterFrame.
     * @param callFromStraterFrame the new callFromStraterFrame value.
     */
    public void setCallFromStraterFrame(final boolean callFromStraterFrame) {
        this.callFromStraterFrame = callFromStraterFrame;
    }

    /**
     * @return gettingJonasFieldFromDiskFrameId.
     */
    public String getGettingJonasFieldFromDiskFrameId() {
        return this.gettingJonasFieldFromDiskFrameId;
    }

    /**
     * Set the gettingJonasFieldFromDiskFrameId.
     * @param gettingJonasFieldFromDiskFrameId the new
     *        gettingJonasFieldFromDiskFrameId value.
     */
    public void setGettingJonasFieldFromDiskFrameId(final String gettingJonasFieldFromDiskFrameId) {
        this.gettingJonasFieldFromDiskFrameId = gettingJonasFieldFromDiskFrameId;
    }

    /**
     * @return gettingJonasFieldFromWebsiteFrameId.
     */
    public String getGettingJonasFieldFromWebsiteFrameId() {
        return this.gettingJonasFieldFromWebsiteFrameId;
    }

    /**
     * Set the gettingJonasFieldFromWebsiteFrameId.
     * @param gettingJonasFieldFromWebsiteFrameId the new
     *        gettingJonasFieldFromWebsiteFrameId value.
     */
    public void setGettingJonasFieldFromWebsiteFrameId(final String gettingJonasFieldFromWebsiteFrameId) {
        this.gettingJonasFieldFromWebsiteFrameId = gettingJonasFieldFromWebsiteFrameId;
    }

    /**
     * @return gettingJonasFieldFromZipFileFrameId.
     */
    public String getGettingJonasFieldFromZipFileFrameId() {
        return this.gettingJonasFieldFromZipFileFrameId;
    }

    /**
     * Set the gettingJonasFieldFromZipFileFrameId.
     * @param gettingJonasFieldFromZipFileFrameId the new
     *        gettingJonasFieldFromZipFileFrameId value.
     */
    public void setGettingJonasFieldFromZipFileFrameId(final String gettingJonasFieldFromZipFileFrameId) {
        this.gettingJonasFieldFromZipFileFrameId = gettingJonasFieldFromZipFileFrameId;
    }

    /**
     * @return licenseFrameId.
     */
    public String getLicenseFrameId() {
        return this.licenseFrameId;
    }

    /**
     * Set the licenseFrameId.
     * @param licenseFrameId the new licenseFrameId value.
     */
    public void setLicenseFrameId(final String licenseFrameId) {
        this.licenseFrameId = licenseFrameId;
    }

    /**
     * @return runningJonasOptionFrameId.
     */
    public String getRunningJonasOptionFrameId() {
        return this.runningJonasOptionFrameId;
    }

    /**
     * Set the runningJonasOptionFrameId.
     * @param runningJonasOptionFrameId the new runningJonasOptionFrameId value.
     */
    public void setRunningJonasOptionFrameId(final String runningJonasOptionFrameId) {
        this.runningJonasOptionFrameId = runningJonasOptionFrameId;
    }

    /**
     * @return the next panel identifier.
     */
    @Override
    public Object getNextPanelDescriptor() {
        this.configuratorFrame.updateBuilderServices();
        this.configuratorFrame.updateBuilderProtocols();
        this.configuratorFrame.updateBuilderWebContainer();
        this.configuratorFrame.updateBuilderJms();
        this.configuratorFrame.updateBuilderHsqlDb();
        getWizard().getConfigButton().setVisible(false);
        getWizard().setConfiguration(this.configuratorFrame.getConfiguration());
        if (this.callFromStraterFrame) {
            return this.runningJonasOptionFrameId;
        }
        return this.buildSummaryFrameId;
    }

    /**
     * @return the back panel identifier.
     */
    @Override
    public Object getBackPanelDescriptor() {
        getWizard().getConfigButton().setVisible(false);
        if (this.callFromStraterFrame) {
            return this.licenseFrameId;
        }
        if (getWizard().getCurrentFrame() == 0) {
            return this.gettingJonasFieldFromWebsiteFrameId;
        } else if (getWizard().getCurrentFrame() == 1) {
            return this.gettingJonasFieldFromDiskFrameId;
        } else if (getWizard().getCurrentFrame() == 2) {
            return this.gettingJonasFieldFromZipFileFrameId;
        }
        return this.gettingJonasFieldFromWebsiteFrameId;
    }

    /**
     * Check if the next button must be enabled.
     */
    @Override
    public void aboutToDisplayPanel() {
        // setNextButtonAccordingToCheckBox();
    }

    /**
     * Browse jdbcDriver folder.
     */
    private void browseJdbcDriver() {
        JFileChooser fc = new JFileChooser();
        fc.setDragEnabled(true);
        if (fc.showOpenDialog(this.configuratorFrame.getJdbcDriverPath()) == JFileChooser.APPROVE_OPTION) {
            File file = fc.getSelectedFile();
            this.configuratorFrame.getJdbcDriverPath().setText(file.getAbsolutePath());
        }
    }

    /**
     * The action performed listener.
     * @param event the event to performed.
     */
    public void actionPerformed(final ActionEvent event) {
        if (event.getSource() == this.configuratorFrame.getUseSsl()) {
            this.configuratorFrame.showSslOption(this.configuratorFrame.getUseSsl().isSelected());
        } else if (event.getSource() == this.configuratorFrame.getCbJrmp()) {
            this.configuratorFrame.showJrmpfields(this.configuratorFrame.getCbJrmp().isSelected());
        } else if (event.getSource() == this.configuratorFrame.getCbIrmi()) {
            this.configuratorFrame.showIrmifields(this.configuratorFrame.getCbIrmi().isSelected());
        } else if (event.getSource() == this.configuratorFrame.getCbIiop()) {
            this.configuratorFrame.showIiopfields(this.configuratorFrame.getCbIiop().isSelected());
        } else if (event.getSource() == this.configuratorFrame.getCbLmi()) {
            this.configuratorFrame.showLmifields(this.configuratorFrame.getCbLmi().isSelected());
        } else if (event.getSource() == this.configuratorFrame.getJbSelectAll()) {
            this.configuratorFrame.selectDeselectAllServices(true);
        } else if (event.getSource() == this.configuratorFrame.getJbDeselectAll()) {
            this.configuratorFrame.selectDeselectAllServices(false);
        } else if (event.getSource() == this.configuratorFrame.getJbAdvancedDsConf()) {
            if (this.configuratorFrame.getDatasourceList().getSelectedValue() != null) {
                int selectedIndex = this.configuratorFrame.getDatasourceList().getSelectedIndex();
                this.configuratorFrame.getAdvancedDatasourceList().setSelectedIndex(selectedIndex);
                this.configuratorFrame.openAdvancedDsConfTabbed();
                String dsName = this.configuratorFrame.getDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.setDatasourceAdvancedParametersValue(dsName);
            } else {
                getWizard().showMessage("Please select a datasource !");
            }
        } else if (event.getSource() == this.configuratorFrame.getJbSaveDatasource()) {
            // Save selected datasource
            checksSaveDatasource();
        } else if (event.getSource() == this.configuratorFrame.getJbSaveAdvancedDsConf()) {
            if (this.configuratorFrame.getAdvancedDatasourceList().getSelectedValue() != null) {
                int selectedIndex = this.configuratorFrame.getAdvancedDatasourceList().getSelectedIndex();
                this.configuratorFrame.getDatasourceList().setSelectedIndex(selectedIndex);
                String dsName = this.configuratorFrame.getAdvancedDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.saveAdvancedDsConfig(dsName);
            }
        } else if (event.getSource() == this.configuratorFrame.getJbResetDatasource()) {
            this.configuratorFrame.resetDatasourceField();
            this.configuratorFrame.enabledJdbcDriverFields();
        } else if (event.getSource() == this.configuratorFrame.getJbResetAdvancedDsConf()) {
            this.configuratorFrame.resetAdvancedConfigField();
        } else if (event.getSource() == this.configuratorFrame.getJbDeleteDatasource()) {
            if (this.configuratorFrame.getDatasourceList().getSelectedValue() != null) {
                int selectedIndex = this.configuratorFrame.getDatasourceList().getSelectedIndex();
                this.configuratorFrame.getDatasourceList().setSelectedIndex(selectedIndex);
                String dsName = this.configuratorFrame.getDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.deleteDatasource(dsName);
            } else {
                getWizard().showMessage("Please select a datasource !");
            }
        } else if (event.getSource() == this.configuratorFrame.getJbBrowseJdbc()) {
            browseJdbcDriver();
        } else if (event.getSource() == this.configuratorFrame.getCopyJdbc()) {
            this.configuratorFrame.enabledJdbcDriverFields();
        } else if (event.getSource() == this.configuratorFrame.getDownloadJdbcFromUrl()) {
            this.configuratorFrame.enabledJdbcDriverFields();
        } else if (event.getSource() == this.configuratorFrame.getDownloadJdbcFromVersion()) {
            this.configuratorFrame.enabledJdbcDriverFields();
        } else if (event.getSource() == this.configuratorFrame.getDbms()) {
            if (this.configuratorFrame.getDbms().getSelectedItem() != null) {
                setJdbcDriverVersion();
            }
        } else if (event.getSource() == this.configuratorFrame.getDriverVersionComboBox()) {
            if (this.configuratorFrame.getDriverVersionComboBox().getSelectedItem() != null) {
                this.configuratorFrame.getDriverVersionTextField().setText(
                        this.configuratorFrame.getDriverVersionComboBox().getSelectedItem().toString());
            }
        } else if (event.getSource() == this.configuratorFrame.getJbAddJmsQueues()) {
            addJmsQueuesToList();
        } else if (event.getSource() == this.configuratorFrame.getJbAddJmsTopics()) {
            addJmsTopicsToList();
        } else if (event.getSource() == this.configuratorFrame.getJbRemoveJmsQueues()) {
            if (this.configuratorFrame.getJmsQueuesList().getSelectedValue() != null) {
                removeJmsQueuesFromList();
            }
        } else if (event.getSource() == this.configuratorFrame.getJbRemoveJmsTopics()) {
            if (this.configuratorFrame.getJmsTopicsList().getSelectedValue() != null) {
                removeJmsTopicsFromList();
            }
        }
    }

    /**
     * The value changed listener.
     * @param event the event to performed.
     */
    public void valueChanged(final ListSelectionEvent event) {
        String dsName = "";
        if (event.getSource() == this.configuratorFrame.getDatasourceList()) {
            if (this.configuratorFrame.getDatasourceList().getSelectedValue() != null) {
                dsName = this.configuratorFrame.getDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.setDatasourceFieldsValue(dsName);
            }
        } else if (event.getSource() == this.configuratorFrame.getAdvancedDatasourceList()) {
            if (this.configuratorFrame.getAdvancedDatasourceList().getSelectedValue() != null) {
                dsName = this.configuratorFrame.getAdvancedDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.setDatasourceAdvancedParametersValue(dsName);
            }
        }
    }

    /**
     * Copy or download JDBC driver to workdirectory.
     * @return true if copy is ok, false otherwise
     */
    private boolean copyJdbcDriverToWorkdirectory() {
        String file = "";
        File workdir = getWizard().getWorkDirectory();
        if (workdir == null) {
            getWizard().getStarter().createTmpWorkDirectory();
        }
        workdir = getWizard().getWorkDirectory();
        if (this.configuratorFrame.getCopyJdbc().isSelected()) {
            String driverPath = this.configuratorFrame.getJdbcDriverPath().getText();
            if (new File(driverPath).exists()) {
                Copy copy = new Copy();
                copy.setFile(new File(driverPath));
                copy.setTodir(workdir);
                copy.execute();
                file = driverPath.substring(driverPath.lastIndexOf(FILE_SEPARATOR) + 1);
            } else {
                return false;
            }
        } else {
            Get get = new Get();
            Project project = new Project();
            String driverUrl = "";
            if (this.configuratorFrame.getDownloadJdbcFromUrl().isSelected()) {
                driverUrl = this.configuratorFrame.getJdbcDriverUrl().getText();

            } else if (this.configuratorFrame.getDownloadJdbcFromVersion().isSelected()) {
                String driverVersion = this.configuratorFrame.getDriverVersionTextField().getText();
                driverUrl = getJdbcUrl(this.configuratorFrame.getDbms().getSelectedItem().toString(), driverVersion);
            }
            try {
                if (Utility.testConnexionURL(new URL(driverUrl))) {
                    try {
                        get.setSrc(new URL(driverUrl));
                    } catch (MalformedURLException ex) {
                        ex.getMessage();
                    }
                    file = driverUrl.substring(driverUrl.lastIndexOf("/") + 1);

                    get.setDest(new File(workdir.getAbsolutePath(), file));
                    get.setProject(project);
                    get.execute();
                } else {
                    return false;
                }
            } catch (MalformedURLException ex) {
                Logger.getLogger(ConfiguratorFrameDescriptor.class.getName()).log(Level.SEVERE, ex.getMessage());
            }
        }
        System.setProperty("dbJdbcDriverPath", workdir.getAbsolutePath() + FILE_SEPARATOR + file);
        return new File(workdir, file).exists();
    }

    /**
     * Returns the jdbc driver url.
     * @param datasourceName the datasource name.
     * @param version the version of jdbc to download.
     * @return the jdbc url.
     */
    private static String getJdbcUrl(final String datasourceName, final String version) {
        /*
         * System.setProperty("version", "stable"); Utility.initProxyValues("",
         * "", "", "");
         */
        Parser parser = null;
        ParseUrl parseurl = null;
        if (datasourceName.toLowerCase().contains("postgresql") || datasourceName.toLowerCase().contains("mysql")) {
            if (datasourceName.toLowerCase().contains("postgresql")) {
                parser = new Parser("http://repo1.maven.org/maven2/" + "postgresql/postgresql/maven-metadata.xml", "", "JOnAS");
                parseurl = new ParseUrl(parser, "http://repo1.maven.org/maven2/" + "postgresql/postgresql/", ".jar");
            } else if (datasourceName.toLowerCase().contains("mysql")) {
                parser = new Parser("http://repo1.maven.org/maven2/org/ow2/" + "jonas/assemblies/maven-metadata.xml", "",
                        "JOnAS");
                parseurl = new ParseUrl(parser, "http://repo1.maven.org/" + "maven2/mysql/mysql-connector-java/", ".jar");
            }
            /*
             * ...
             */
            return parseurl.getUrl(version);
        }
        // Others datasource cannot be downloaded through the ParseUrl object.
        return "";
    }

    /**
     * Set the JdbcDriverVersion combobox.
     */
    private void setJdbcDriverVersion() {
        String dbms = this.configuratorFrame.getDbms().getSelectedItem().toString().toLowerCase();
        if (dbms.equals("postgresql") || dbms.equals("mysql")) {
            Parser parser = null;
            if (dbms.contains("postgresql")) {
                parser = new Parser("http://repo1.maven.org/maven2/" + "postgresql/postgresql/maven-metadata.xml", "", "JOnAS");
            } else if (dbms.contains("mysql")) {
                parser = new Parser("http://repo1.maven.org/maven2/" + "mysql/mysql-connector-java/" + "maven-metadata.xml",
                        "", "JOnAS");
            }
            this.configuratorFrame.getDriverVersionComboBox().removeAllItems();
            for (String version : parser.getAllLatestVersions()) {
                this.configuratorFrame.getDriverVersionComboBox().addItem(version);
            }
            this.configuratorFrame.getDriverVersionComboBox().setSelectedIndex(
                    this.configuratorFrame.getDriverVersionComboBox().getItemCount() - 1);
            this.configuratorFrame.getDriverVersionTextField().setText(
                    this.configuratorFrame.getDriverVersionComboBox().getSelectedItem().toString());
        } else {
            this.configuratorFrame.getDriverVersionComboBox().removeAllItems();
            this.configuratorFrame.getDriverVersionTextField().setText("");
        }
        // No item is selected, then set fields
        if (this.configuratorFrame.getDatasourceList().getSelectedValue() == null) {
            String datasourceUrl = this.configuratorFrame.getDatasourceDefaultUrl(this.configuratorFrame.getDbms().getSelectedItem()
                    .toString());
            String className = this.configuratorFrame.getDatasourceDefaultClassName(this.configuratorFrame.getDbms().getSelectedItem()
                    .toString());
            String mapper = this.configuratorFrame.getDatasourceDefaultMapper(this.configuratorFrame.getDbms().getSelectedItem()
                    .toString());
            this.configuratorFrame.getDatasourceUrl().setText(datasourceUrl);
            this.configuratorFrame.getDatasourceClassName().setText(className);
            this.configuratorFrame.getDatasourceMapper().setText(mapper);
        }
    }

    /**
     * @return the driver path and version.
     */
    private String[] getJdbcDriver() {
        String driverPath = "";
        String driverVersion = "";
        if (this.configuratorFrame.getCopyJdbc().isSelected()) {
            driverPath = this.configuratorFrame.getJdbcDriverPath().getText();
        } else if (this.configuratorFrame.getDownloadJdbcFromUrl().isSelected()) {
            driverPath = this.configuratorFrame.getJdbcDriverUrl().getText();
        } else if (this.configuratorFrame.getDownloadJdbcFromVersion().isSelected()) {
            driverVersion = this.configuratorFrame.getDriverVersionTextField().getText();
        }
        return new String[] {driverPath, driverVersion};
    }

    /**
     * Checks the fields and save the datasource.
     */
    private void checksSaveDatasource() {
        String driverPath = getJdbcDriver()[0];
        String driverVersion = getJdbcDriver()[1];
        String dsName = this.configuratorFrame.getDatasourceName().getText();
        String jndiName = this.configuratorFrame.getJndiName().getText();
        String rarName = this.configuratorFrame.getRaName().getText();
        String className = this.configuratorFrame.getDatasourceClassName().getText();
        String dsUrl = this.configuratorFrame.getDatasourceUrl().getText();
        String dsMapper = this.configuratorFrame.getDatasourceMapper().getText();
        String dsUser = this.configuratorFrame.getUser().getText();
        String rarLink = (String) this.configuratorFrame.getRarLink().getSelectedItem();
        boolean exists = this.configuratorFrame.getJdbcDatasources().contains(dsName);
        String dbmsIndex = (String) this.configuratorFrame.getDbms().getSelectedItem();
        boolean isOk = true;
        if (dsName.equals("")) {
            getWizard().showMessage("Set the datasource name !");
            isOk = false;
        } else if (jndiName.equals("")) {
            getWizard().showMessage("Set the jndi name !");
            isOk = false;
        } else if (className.equals("")) {
            getWizard().showMessage("Set the class name !");
            isOk = false;
        } else if (dsMapper.equals("")) {
            getWizard().showMessage("Set the mapper !");
            isOk = false;
        } else if (dsUrl.equals("")) {
            getWizard().showMessage("Set the URL !");
            isOk = false;
        } else if (dsUrl.contains("<") || dsUrl.contains(">")) {
            getWizard().showMessage("Database URL contains \"<\" or \">\"." + "\nPlease set a valid URL !");
            isOk = false;
        } else if (dsUser.equals("")) {
            getWizard().showMessage("Set the user name !");
            isOk = false;
        } else if (rarLink.equals("")) {
            getWizard().showMessage("Select the RAR Link !");
            isOk = false;
        } else if (dbmsIndex.equals("")) {
            getWizard().showMessage("Please select the DBMS !");
            isOk = false;
        } else if (this.configuratorFrame.getCopyJdbc().isSelected() && driverPath.equals("")) {
            getWizard().showMessage("Please set the driver JAR file !");
            isOk = false;
        } else if (this.configuratorFrame.getDownloadJdbcFromUrl().isSelected() && driverPath.equals("")) {
            getWizard().showMessage("Please set the driver URL !");
            isOk = false;
        } else if (this.configuratorFrame.getDownloadJdbcFromVersion().isSelected() && driverVersion.equals("")) {
            getWizard().showMessage("Please set the driver version !");
            isOk = false;
        } else {
            isOk = true;
        }
        if (rarName.equals("")) {
            this.configuratorFrame.getRaName().setText(dsName);
        }
        if (isOk) {
            copyJdbcDriverToWorkdirectory();
            if (this.configuratorFrame.getDatasourceList().getSelectedValue() != null) {
                int selectedIndex = this.configuratorFrame.getDatasourceList().getSelectedIndex();
                this.configuratorFrame.getDatasourceList().setSelectedIndex(selectedIndex);
                dsName = this.configuratorFrame.getDatasourceList().getSelectedValue().toString();
                this.configuratorFrame.updateBuilderDatasources(dsName);
            } else {
                if (exists) {
                    getWizard().showMessage("Datasource \"" + dsName + "\"already exists!");
                } else {
                    this.configuratorFrame.updateBuilderDatasources(null);
                }
            }
            this.configuratorFrame.resetDatasourceField();
        }
    }

    /**
     * Add a new jms queue to the jms queue list.
     */
    private void addJmsQueuesToList() {
        String queue = this.configuratorFrame.getJmsQueues().getText();
        if (!queue.equals("") && !this.configuratorFrame.getJmsQueuesItems().contains(queue)) {
            this.configuratorFrame.getJmsQueuesItems().add(queue);
            Collections.sort(this.configuratorFrame.getJmsQueuesItems());
            this.configuratorFrame.getJmsQueuesList().setListData(new Vector<String>(this.configuratorFrame.getJmsQueuesItems()));
            this.configuratorFrame.getJmsQueues().setText("");
        }
    }

    /**
     * Add a new jms topic to the jms topics list.
     */
    private void addJmsTopicsToList() {
        String topic = this.configuratorFrame.getJmsTopics().getText();
        if (!topic.equals("") && !this.configuratorFrame.getJmsTopicsItems().contains(topic)) {
            this.configuratorFrame.getJmsTopicsItems().add(topic);
            Collections.sort(this.configuratorFrame.getJmsTopicsItems());
            this.configuratorFrame.getJmsTopicsList().setListData(new Vector<String>(this.configuratorFrame.getJmsTopicsItems()));
            this.configuratorFrame.getJmsTopics().setText("");
        }
    }

    /**
     * Remove the queue from the jms queue list.
     */
    private void removeJmsQueuesFromList() {
        String queue = (String) this.configuratorFrame.getJmsQueuesList().getSelectedValue();
        this.configuratorFrame.getJmsQueuesItems().remove(queue);
        this.configuratorFrame.getJmsQueuesList().setListData(new Vector<String>(this.configuratorFrame.getJmsQueuesItems()));
    }

    /**
     * Remove the topic from the jms topic list.
     */
    private void removeJmsTopicsFromList() {
        String topic = (String) this.configuratorFrame.getJmsTopicsList().getSelectedValue();
        this.configuratorFrame.getJmsTopicsItems().remove(topic);
        this.configuratorFrame.getJmsTopicsList().setListData(new Vector<String>(this.configuratorFrame.getJmsTopicsItems()));
    }
}
